" FILE: Lgrep.vim
" AUTHOR:  yuratomo
" Last Modified: 07 Feb 2012.
" Version: 0.0.3
" 
" [USAGE]
"
"  grep start:
"   :Lgrep /s keyword filepattern
"
"  grep cancel:
"   :LgrepCancel
"
"  show grep status:
"   :LgrepStatus
"
"  update quickfix list:
"   :LgrepUpdate
"

if exists('g:Lgrep_loaded') && g:Lgrep_loaded == 1
  finish
endif
if !exists('g:Lgrep#AutoUpdateQuickFix')
  let g:Lgrep#AutoUpdateQuickFix=1
endif

command! -nargs=* Lgrep :call Lgrep#DoGrep(<f-args>)
command! -nargs=0 LgrepUpdate :call Lgrep#UpdateQuickFix()
command! -nargs=0 LgrepCancel :call Lgrep#CancelGrep()
command! -nargs=0 LgrepStatus :call Lgrep#ShowStatus()

let g:Lgrep#status = 'no start'
let g:Lgrep#findNum = 0
let g:Lgrep#last_file = ''

function! Lgrep#DoGrep(...)
  let g:Lgrep#last_file = getcwd()
  let g:Lgrep#temp_file = tempname()
  let g:Lgrep#findNum = 0
  silent exe '!start /MIN cmd /c TITLE Lgrep'.v:servername.' & findstr /n '.join(a:000,' ').'>'.g:Lgrep#temp_file.' & '.$vim.'\gvim --cmd "call remote_expr('."'".v:servername."','"."GrepPostProc()"."'".')" --cmd ":q" --'
  let g:Lgrep#status = 'processing now'
endfunction

function! GrepPostProc()
  let g:Lgrep#status = 'grep end'
  if g:Lgrep#AutoUpdateQuickFix==1
    call Lgrep#UpdateQuickFix()
  endif
  call Lgrep#ShowStatus()
endfunction

function! Lgrep#UpdateQuickFix()
  if exists('g:Lgrep#temp_file')
    exe 'cd '.g:Lgrep#last_file
    exe 'cgetfile '.g:Lgrep#temp_file
    cw
    if g:Lgrep#status == 'grep end'
      call Lgrep#ClearTempFile()
    endif
  endif
endfunction

function! Lgrep#CancelGrep()
  silent exe '!start /b cmd /c taskkill /T /F /FI "WINDOWTITLE eq Lgrep'.v:servername.'" & taskkill /T /F /FI "WINDOWTITLE eq Ǘ:  Lgrep'.v:servername.'*"'
  let g:Lgrep#status = 'canceled'
  if g:Lgrep#AutoUpdateQuickFix==1
    call Lgrep#UpdateQuickFix()
  endif
  call Lgrep#ShowStatus()
endfunction

function! Lgrep#ShowStatus()
  if exists('g:Lgrep#temp_file')
    let g:Lgrep#findNum=split(tr(system('find /C /V "" '.g:Lgrep#temp_file),"\n",' '),' ')[2]
  endif
  echo g:Lgrep#findNum." items fouond.(".g:Lgrep#status.")"
endfunction

function Lgrep#ClearTempFile()
  if exists('g:Lgrep#temp_file')
    call delete(g:Lgrep#temp_file)
    unlet g:Lgrep#temp_file
  endif
endfunction

let g:Lgrep_loaded = 1
